@@ -1,8 +1,6 @@
-import binascii
 import glob
 import logging
 import os
-import re
 import string
 import struct
 import zlib
@@ -10,6 +8,8 @@ from pathlib import Path
 
 import vdf
 
+from .util import lower_dict
+
 __all__ = (
     "COMMON_STEAM_DIRS", "SteamApp", "find_steam_path",
     "find_steam_proton_app", "find_proton_app", "find_steam_runtime_path",
@@ -104,21 +104,21 @@ class SteamApp(object):
                 # eg. due to running a Linux filesystem under Windows
                 # In that case just skip it
                 logger.warning(
-                    "Skipping malformed appmanifest {}".format(path)
+                    "Skipping malformed appmanifest %s", path
                 )
                 return None
 
         try:
             vdf_data = vdf.loads(content)
         except SyntaxError:
-            logger.warning("Skipping malformed appmanifest {}".format(path))
+            logger.warning("Skipping malformed appmanifest %s", path)
             return None
 
         try:
             app_state = vdf_data["AppState"]
         except KeyError:
             # Some appmanifest files may be empty. Ignore those.
-            logger.info("Skipping empty appmanifest {}".format(path))
+            logger.info("Skipping empty appmanifest %s", path)
             return None
 
         # The app ID field can be named 'appID' or 'appid'.
@@ -126,7 +126,7 @@ class SteamApp(object):
         # files (created by old Steam clients?) also use 'appID'.
         #
         # Use case-insensitive field names to deal with these.
-        app_state = {k.lower(): v for k, v in app_state.items()}
+        app_state = lower_dict(app_state)
         appid = int(app_state["appid"])
         name = app_state["name"]
 
@@ -196,8 +196,9 @@ def find_steam_path():
         steam_path = str(Path.home() / steam_path)
         if has_steamapps_dir(steam_path):
             logger.info(
-                "Found Steam directory at {}. You can also define Steam "
-                "directory manually using $STEAM_DIR".format(steam_path)
+                "Found Steam directory at %s. You can also define Steam "
+                "directory manually using $STEAM_DIR",
+                steam_path
             )
             if not steam_root:
                 steam_root = steam_path
@@ -412,7 +413,7 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
     try:
         app = next(app for app in steam_apps if app.name == compat_tool_name)
         logger.info(
-            "Found active custom Proton installation: {}".format(app.name)
+            "Found active custom Proton installation: %s", app.name
         )
         return app
     except StopIteration:
@@ -431,7 +432,7 @@ def find_steam_proton_app(steam_path, steam_apps, appid=None):
     try:
         app = next(app for app in steam_apps if app.appid == proton_appid)
         logger.info(
-            "Found active Proton installation: {}".format(app.name)
+            "Found active Proton installation: %s", app.name
         )
         return app
     except StopIteration:
@@ -499,7 +500,7 @@ def find_proton_app(steam_path, steam_apps, appid=None):
             proton_app = next(
                 app for app in steam_apps if app.name == proton_version)
             logger.info(
-                 "Found requested Proton version: {}".format(proton_app.name)
+                "Found requested Proton version: %s", proton_app.name
             )
             return proton_app
         except StopIteration:
@@ -537,7 +538,7 @@ def get_steam_lib_paths(steam_path):
         ]
 
         logger.info(
-            "Found {} Steam library folders".format(len(library_folders))
+            "Found %d Steam library folders", len(library_folders)
         )
         return library_folders
 
@@ -561,6 +562,10 @@ def get_steam_lib_paths(steam_path):
 
 
 def get_compat_tool_dirs(steam_root):
+    """
+    Return a list of compatibility tool directories in order from
+    directories with lowest precedence
+    """
     # The path list is ordered by priority, starting from Proton apps
     # with the lowest precedence ('/usr/share/steam/compatibilitytools.d')
     paths = [
@@ -584,10 +589,12 @@ def get_proton_installations(compat_tool_dir):
         return []
 
     comptool_files = glob.glob(
-        os.path.join(compat_tool_dir, "*", "compatibilitytool.vdf")
+        os.path.join(
+            glob.escape(compat_tool_dir), "*", "compatibilitytool.vdf"
+        )
     )
     comptool_files += glob.glob(
-        os.path.join(compat_tool_dir, "compatibilitytool.vdf")
+        os.path.join(glob.escape(compat_tool_dir), "compatibilitytool.vdf")
     )
 
     custom_proton_apps = []
@@ -640,6 +647,9 @@ def get_custom_proton_installations(steam_root):
 
 
 def find_current_steamid3(steam_path):
+    """
+    Find the SteamID3 of the currently logged in Steam user
+    """
     def to_steamid3(steamid64):
         """Convert a SteamID64 into the SteamID3 format"""
         return int(steamid64) & 0xffffffff
@@ -652,13 +662,17 @@ def find_current_steamid3(steam_path):
     except IOError:
         return None
 
+    user_datas = [
+        (user_id, lower_dict(user_data))
+        for user_id, user_data in vdf_data["users"].items()
+    ]
     users = [
         {
             "steamid3": to_steamid3(user_id),
-            "account_name": user_data["AccountName"],
-            "timestamp": user_data.get("Timestamp", 0)
+            "account_name": user_data["accountname"],
+            "timestamp": user_data.get("timestamp", 0)
         }
-        for user_id, user_data in vdf_data["users"].items()
+        for user_id, user_data in user_datas
     ]
 
     # Return the user with the highest timestamp, as that's likely to be the
@@ -718,7 +732,7 @@ def get_custom_windows_shortcuts(steam_path):
     for shortcut_id, shortcut_data in vdf_data["shortcuts"].items():
         # The "exe" field can also be "Exe". Account for this by making
         # all field names lowercase
-        shortcut_data = {k.lower(): v for k, v in shortcut_data.items()}
+        shortcut_data = lower_dict(shortcut_data)
         shortcut_id = int(shortcut_id)
 
         appid = get_appid_from_shortcut(
@@ -757,13 +771,31 @@ def get_steam_apps(steam_root, steam_path, steam_lib_paths):
 
     for path in steam_lib_paths:
         appmanifest_paths = []
-        if os.path.isdir(os.path.join(path, "steamapps")):
+        is_lowercase = os.path.isdir(os.path.join(path, "steamapps"))
+        is_mixedcase = os.path.isdir(os.path.join(path, "SteamApps"))
+
+        if is_lowercase:
             appmanifest_paths = glob.glob(
-                os.path.join(path, "steamapps", "appmanifest_*.acf")
+                os.path.join(
+                    glob.escape(path), "steamapps", "appmanifest_*.acf"
+                )
             )
-        elif os.path.isdir(os.path.join(path, "SteamApps")):
+        elif is_mixedcase:
             appmanifest_paths = glob.glob(
-                os.path.join(path, "SteamApps", "appmanifest_*.acf")
+                os.path.join(
+                    glob.escape(path), "SteamApps", "appmanifest_*.acf"
+                )
+            )
+
+        if is_lowercase and is_mixedcase:
+            # Log a warning if both 'steamapps' and 'SteamApps' directories
+            # exist, as both Protontricks and Steam client have problems
+            # dealing with it (see issue #51)
+            logger.warning(
+                "Both 'steamapps' and 'SteamApps' directories were found at "
+                "%s. 'SteamApps' directory should be removed to prevent "
+                "issues with app and Proton discovery.",
+                path
             )
 
         for manifest_path in appmanifest_paths:
